/* initng-conf-gtk, the gtk configuration program for initng
 * Copyright (C) 2006 Daniel Malmgren <dm@mensa.se>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */
 
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <string.h>

#include "rlparser.h"
#include "serviceparser.h"
#include "ngchandler.h"
#include "initng-conf-gtk_globals.h"
#include "gconf.h"

gboolean cleanup_and_quit() {
  //Add code to clean up stuff and free memory here!!!
  GtkWidget *initng_conf_gtk_app, *dialog, *save_button;

  if(!xml)
    fprintf(stderr, "xml is null!\n");

  initng_conf_gtk_app = glade_xml_get_widget(xml, "initng_conf_gtk_app");
  save_button = glade_xml_get_widget(xml, "save_button");
  
  if(GTK_WIDGET_IS_SENSITIVE(save_button)) {
    dialog = gtk_message_dialog_new (GTK_WINDOW(initng_conf_gtk_app),
                                     GTK_DIALOG_MODAL,
                                     GTK_MESSAGE_QUESTION,
                                     GTK_BUTTONS_OK_CANCEL,
                                     _("Quit without saving changes?"));
    if(gtk_dialog_run(GTK_DIALOG (dialog)) == GTK_RESPONSE_OK) {
      servicelist_cleanup();
      gtk_main_quit();
      return FALSE;
    }
    gtk_widget_destroy (dialog);
    return TRUE;
  } else {
    servicelist_cleanup();
    gtk_main_quit();
    return FALSE;  
  }
}

void on_save_activate(GtkMenuItem *menuitem, gpointer user_data) {
  GtkWidget *save_button = glade_xml_get_widget(xml, "save_button");
  GtkWidget *save1 = glade_xml_get_widget(xml, "save1");
  GtkWidget *statusbar = glade_xml_get_widget(xml, "statusbar");
  guint context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar), "");
  save_rl();
  save_all_services();
  gtk_statusbar_pop(GTK_STATUSBAR(statusbar), context_id);
  gtk_statusbar_push(GTK_STATUSBAR(statusbar), context_id, _("All changes saved."));
  gtk_widget_set_sensitive(save_button, FALSE);
  gtk_widget_set_sensitive(save1, FALSE);
}

void on_about_activate(GtkMenuItem *menuitem, gpointer user_data) {
  const gchar *authors[] = {
    	"Daniel Malmgren <dm@mensa.se>",
    	NULL
  };
  
  GtkWidget *initng_conf_gtk_app = glade_xml_get_widget(xml, "initng_conf_gtk_app");

  gtk_show_about_dialog (GTK_WINDOW (initng_conf_gtk_app),
			 "name", "initng-conf-gtk",
                         "comments", _("Configuration and control utility for InitNG"),
                         "version", VERSION,
                         "copyright", "Copyright \xc2\xa9 2003-2007 Daniel Malmgren",
                         "authors", authors,
                         "translator-credits", 
			 _("Grzegorz Dabrowski\nDaniel Malmgren\nGuillaume Pujol\nDenis Knauf"),
                         "logo-icon-name", "initng",
                         "wrap-license", TRUE,
                         "website", "http://www.initng.org/wiki/initng-conf-gtk",
                         NULL);

}

void on_runlevel_combo_changed(GtkComboBox *combobox, gpointer user_data) {
  update_rllist(GTK_WIDGET(combobox));
}

void on_add_button_clicked(GtkButton *button, gpointer user_data) {
  GtkWidget *service_list;
  GtkTreeIter serviceiter;
  GtkTreeSelection *selected;
  GtkTreeModel *servicemodel;
  struct s_info *serviceinfo;

  service_list = glade_xml_get_widget(xml, "service_list");
  selected = gtk_tree_view_get_selection(GTK_TREE_VIEW(service_list));
  gtk_tree_selection_get_selected(selected, &servicemodel, &serviceiter);
  gtk_tree_model_get (servicemodel, &serviceiter, 4, &serviceinfo, -1);

  add_to_rl(serviceinfo->fullname);
  gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
}

void on_remove_button_clicked(GtkButton *button, gpointer user_data) {
  remove_from_rl();
  gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
}

void on_service_list_cursor_changed(GtkTreeView *treeview, gpointer user_data) {
  GtkTreeView *runlevel_list = GTK_TREE_VIEW(glade_xml_get_widget(xml,
								  "runlevel_list"));
  GtkTreeIter service_iter, runlevel_iter;
  GtkTreeModel *service_model, *runlevel_model;
  GtkWidget *add_button = glade_xml_get_widget(xml, "add_button");

  GtkWidget *start_button = glade_xml_get_widget(xml, "start_button");
  GtkWidget *stop_button = glade_xml_get_widget(xml, "stop_button");
  GtkWidget *restart_button = glade_xml_get_widget(xml, "restart_button");
  GtkWidget *zap_button = glade_xml_get_widget(xml, "zap_button");
  GtkWidget *edit_button = glade_xml_get_widget(xml, "edit_button");
  GtkWidget *start1 = glade_xml_get_widget(xml, "start1");
  GtkWidget *stop1 = glade_xml_get_widget(xml, "stop1");
  GtkWidget *restart1 = glade_xml_get_widget(xml, "restart1");
  GtkWidget *zap1 = glade_xml_get_widget(xml, "zap1");
  GtkWidget *edit1 = glade_xml_get_widget(xml, "edit1");
  GtkTreeSelection *selected = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
  struct s_info *serviceinfo;

  gtk_tree_selection_get_selected(selected, &service_model, &service_iter);
  gtk_tree_model_get(service_model, &service_iter, 4, &serviceinfo, -1);

  if(serviceinfo) {
    switch(serviceinfo->status_id) {
    case IS_UP:
      gtk_widget_set_sensitive(start_button, FALSE);
      gtk_widget_set_sensitive(stop_button, TRUE);
      gtk_widget_set_sensitive(restart_button, TRUE);
      gtk_widget_set_sensitive(zap_button, TRUE);
      gtk_widget_set_sensitive(edit_button, TRUE);
      gtk_widget_set_sensitive(start1, FALSE);
      gtk_widget_set_sensitive(stop1, TRUE);
      gtk_widget_set_sensitive(restart1, TRUE);
      gtk_widget_set_sensitive(zap1, TRUE);
      gtk_widget_set_sensitive(edit1, TRUE);
      break;
    case IS_DOWN:
      gtk_widget_set_sensitive(start_button, TRUE);
      gtk_widget_set_sensitive(stop_button, FALSE);
      gtk_widget_set_sensitive(restart_button, FALSE);
      gtk_widget_set_sensitive(zap_button, TRUE);
      gtk_widget_set_sensitive(edit_button, TRUE);
      gtk_widget_set_sensitive(start1, TRUE);
      gtk_widget_set_sensitive(stop1, FALSE);
      gtk_widget_set_sensitive(restart1, FALSE);
      gtk_widget_set_sensitive(zap1, TRUE);
      gtk_widget_set_sensitive(edit1, TRUE);
      break;
    case IS_FAILED:
    case IS_STARTING:
    case IS_STOPPING:
    case IS_WAITING:
      gtk_widget_set_sensitive(start_button, FALSE);
      gtk_widget_set_sensitive(stop_button, FALSE);
      gtk_widget_set_sensitive(restart_button, FALSE);
      gtk_widget_set_sensitive(zap_button, TRUE);
      gtk_widget_set_sensitive(edit_button, TRUE);
      gtk_widget_set_sensitive(start1, FALSE);
      gtk_widget_set_sensitive(stop1, FALSE);
      gtk_widget_set_sensitive(restart1, FALSE);
      gtk_widget_set_sensitive(zap1, TRUE);
      gtk_widget_set_sensitive(edit1, TRUE);
      break;
    default:
      gtk_widget_set_sensitive(start_button, TRUE);
      gtk_widget_set_sensitive(stop_button, FALSE);
      gtk_widget_set_sensitive(restart_button, FALSE);
      gtk_widget_set_sensitive(zap_button, FALSE);
      gtk_widget_set_sensitive(edit_button, TRUE);
      gtk_widget_set_sensitive(start1, TRUE);
      gtk_widget_set_sensitive(stop1, FALSE);
      gtk_widget_set_sensitive(restart1, FALSE);
      gtk_widget_set_sensitive(zap1, FALSE);
      gtk_widget_set_sensitive(edit1, TRUE);
      break;
    }
    if(!serviceinfo->file_content) {
      gtk_widget_set_sensitive(edit_button, FALSE);
    }
    runlevel_model = gtk_tree_view_get_model(runlevel_list);
    if(runlevel_model) {
      gboolean valid = gtk_tree_model_get_iter_first(runlevel_model, &runlevel_iter);
      
      while(valid) {
	gchar *str_data;
	gtk_tree_model_get(runlevel_model, &runlevel_iter, 0, &str_data, -1);
	if(!strcmp(serviceinfo->fullname, str_data)) {
	  g_free(str_data);
	  break;
	}
	g_free(str_data);
	valid = gtk_tree_model_iter_next(runlevel_model, &runlevel_iter);
      }
      //Only enable the add button if chosen service wasn't
      //already found in runlevel list
      gtk_widget_set_sensitive(add_button, !valid);
    }
  } else {
    gtk_widget_set_sensitive(start_button, FALSE);
    gtk_widget_set_sensitive(stop_button, FALSE);
    gtk_widget_set_sensitive(restart_button, FALSE);
    gtk_widget_set_sensitive(zap_button, FALSE);
    gtk_widget_set_sensitive(edit_button, FALSE);
    gtk_widget_set_sensitive(add_button, FALSE);
  }
}

void on_runlevel_list_cursor_changed(GtkTreeView *treeview, gpointer user_data) {
  GtkWidget *remove_button;

  remove_button = glade_xml_get_widget(xml, "remove_button");

  gtk_widget_set_sensitive(remove_button, TRUE);
}

void on_new_runlevel_button_clicked(GtkButton *button, gpointer user_data) {
  xml_new_runlevel = glade_xml_new(FILENAME, "new_runlevel", NULL);
  if(!xml_new_runlevel)
    xml_new_runlevel = glade_xml_new(SECONDARY_FILENAME,
				     "new_runlevel", NULL);
  glade_xml_signal_autoconnect(xml_new_runlevel);
}

void on_cancel_new_runlevel_button_clicked(GtkButton *button, gpointer user_data) {
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

void on_ok_new_runlevel_button_clicked(GtkButton *button, gpointer user_data) {
  GtkWidget *name_entry;
  name_entry = glade_xml_get_widget(xml_new_runlevel, "new_runlevel_name_entry");
  new_runlevel(gtk_entry_get_text(GTK_ENTRY(name_entry)));
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

void on_cancel_start_service_button_clicked(GtkButton *button, gpointer user_data) {
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

void on_ok_start_service_button_clicked (GtkButton *button, gpointer user_data)
{
  GtkWidget *name_entry;
  name_entry = glade_xml_get_widget(xml_start_service, "start_service_name_entry");
  start_service(gtk_entry_get_text(GTK_ENTRY(name_entry)));
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

void on_cancel_add_service_button_clicked(GtkButton *button, gpointer user_data) {
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

void on_ok_add_service_button_clicked(GtkButton *button, gpointer user_data) {
  GtkWidget *name_entry;
  name_entry = glade_xml_get_widget(xml_add_service, "add_service_name_entry");
  add_to_rl(gtk_entry_get_text(GTK_ENTRY(name_entry)));
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

void on_cancel_edit_service_button_clicked(GtkButton *button, gpointer user_data) {
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

void on_ok_edit_service_button_clicked(GtkButton *button, gpointer user_data) {
  GtkWidget *save_button = glade_xml_get_widget(xml, "save_button");
  GtkWidget *save1 = glade_xml_get_widget(xml, "save1");
  gtk_widget_set_sensitive(save_button, TRUE);
  gtk_widget_set_sensitive(save1, TRUE);
  change_service();
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

void on_apply_edit_service_button_clicked(GtkButton *button, gpointer user_data) {
  GtkWidget *save_button = glade_xml_get_widget(xml, "save_button");
  GtkWidget *save1 = glade_xml_get_widget(xml, "save1");
  gtk_widget_set_sensitive(save_button, TRUE);
  gtk_widget_set_sensitive(save1, TRUE);
  change_service();
}

void on_show_runlevel_editor_activate(GtkMenuItem *menuitem, gpointer user_data) {
  GtkCheckMenuItem *show_runlevel_editor = GTK_CHECK_MENU_ITEM(menuitem);
  GtkWidget *runlevel_editor = glade_xml_get_widget(xml, "runlevel_editor");
  GtkWidget *initng_conf_gtk_app = glade_xml_get_widget(xml, "initng_conf_gtk_app");
  GtkWidget *runlevel_combo = glade_xml_get_widget(xml, "runlevel_combo");
  if(gtk_check_menu_item_get_active(show_runlevel_editor)) {
    GtkWidget *add_button = glade_xml_get_widget(xml, "add_button");
    GtkTreeView *service_list = GTK_TREE_VIEW(glade_xml_get_widget(xml,
								   "service_list"));
    GtkTreeSelection *selected = gtk_tree_view_get_selection(service_list);
    if(gtk_tree_selection_get_selected(selected, NULL, NULL))
      gtk_widget_set_sensitive(add_button, TRUE);
    gtk_widget_show_all(runlevel_editor);
    set_gconf_bool(GCONF_SHOW_RUNLEVEL_EDITOR, TRUE);
    if(gtk_combo_box_get_active(GTK_COMBO_BOX(runlevel_combo)) < 0)
      gtk_combo_box_set_active(GTK_COMBO_BOX(runlevel_combo), 0);
  } else {
    gint window_height;
    gtk_widget_hide_all(runlevel_editor);
    set_gconf_bool(GCONF_SHOW_RUNLEVEL_EDITOR, FALSE);
    gtk_window_get_size(GTK_WINDOW(initng_conf_gtk_app), NULL, &window_height);
    gtk_window_resize(GTK_WINDOW(initng_conf_gtk_app), 1, window_height);
  }
}

void on_show_advanced_activate(GtkMenuItem *menuitem, gpointer user_data) {
  GtkCheckMenuItem *show_advanced = GTK_CHECK_MENU_ITEM(menuitem);
  GtkWidget *zap_button = glade_xml_get_widget(xml, "zap_button");
  GtkWidget *edit_button = glade_xml_get_widget(xml, "edit_button");
  if(gtk_check_menu_item_get_active(show_advanced)) {
    gtk_widget_show(zap_button);
    gtk_widget_show(edit_button);
    set_gconf_bool(GCONF_SHOW_ADVANCED, TRUE);
  } else {
    gtk_widget_hide(zap_button);
    gtk_widget_hide(edit_button);
    set_gconf_bool(GCONF_SHOW_ADVANCED, FALSE);
  }
}

void on_show_toolbar_activate(GtkMenuItem *menuitem, gpointer user_data) {
  GtkCheckMenuItem *show_toolbar = GTK_CHECK_MENU_ITEM(menuitem);
  GtkWidget *toolbar = glade_xml_get_widget(xml, "toolbar");
  if(gtk_check_menu_item_get_active(show_toolbar)) {
    gtk_widget_show(toolbar);
    set_gconf_bool(GCONF_SHOW_TOOLBAR, TRUE);
  } else {
    gtk_widget_hide(toolbar);
    set_gconf_bool(GCONF_SHOW_TOOLBAR, FALSE);
  }
}

void on_show_statusbar_activate(GtkMenuItem *menuitem, gpointer user_data) {
  GtkCheckMenuItem *show_statusbar = GTK_CHECK_MENU_ITEM(menuitem);
  GtkWidget *statusbar = glade_xml_get_widget(xml, "statusbar");
  if(gtk_check_menu_item_get_active(show_statusbar)) {
    gtk_widget_show(statusbar);
    set_gconf_bool(GCONF_SHOW_STATUSBAR, TRUE);
  } else {
    gtk_widget_hide(statusbar);
    set_gconf_bool(GCONF_SHOW_STATUSBAR, FALSE);
  }
}

void on_service_list_row_activated(GtkTreeView *treeview, GtkTreePath *path,
				   GtkTreeViewColumn *column, gpointer user_data) {
  GtkWidget *edit_button = glade_xml_get_widget(xml, "edit_button");
  if(GTK_WIDGET_SENSITIVE(edit_button)) {
    GtkWidget *ok_edit_service_button, *apply_edit_service_button;
    xml_show_service = glade_xml_new(FILENAME, "show_service", NULL);
    if(!xml_show_service)
      xml_show_service = glade_xml_new(SECONDARY_FILENAME, "show_service", NULL);
    
    ok_edit_service_button = glade_xml_get_widget(xml_show_service,
						  "ok_edit_service_button");
    apply_edit_service_button = glade_xml_get_widget(xml_show_service, 
						     "apply_edit_service_button");
    
    if(show_service(GTK_WIDGET(treeview))) {
      gtk_widget_set_sensitive(ok_edit_service_button, TRUE);
      gtk_widget_set_sensitive(apply_edit_service_button, TRUE);
    } else {
      gtk_widget_set_sensitive(ok_edit_service_button, FALSE);
      gtk_widget_set_sensitive(apply_edit_service_button, FALSE);
    }
    
    GtkWidget *show_service = glade_xml_get_widget(xml_show_service, "show_service");
    
    glade_xml_signal_autoconnect(xml_show_service);
  }
}

void on_stop_clicked(GtkToolButton *toolbutton, gpointer user_data) {
  GtkWidget *service_list = glade_xml_get_widget(xml, "service_list");
  GtkTreeIter serviceiter;
  GtkTreeSelection *selected;
  GtkTreeModel *servicemodel;
  struct s_info *serviceinfo;

  selected = gtk_tree_view_get_selection(GTK_TREE_VIEW(service_list));
  if(gtk_tree_selection_get_selected(selected, &servicemodel, &serviceiter)) {
    gtk_tree_model_get (servicemodel, &serviceiter, 4, &serviceinfo, -1);
    stop_service(serviceinfo->fullname);
  }
}

void on_start_clicked(GtkToolButton *toolbutton, gpointer user_data) {
  GtkWidget *service_list = glade_xml_get_widget(xml, "service_list");
  GtkTreeIter serviceiter;
  GtkTreeSelection *selected;
  GtkTreeModel *servicemodel;
  struct s_info *serviceinfo;

  selected = gtk_tree_view_get_selection(GTK_TREE_VIEW(service_list));
  if(gtk_tree_selection_get_selected(selected, &servicemodel, &serviceiter)) {
    gtk_tree_model_get (servicemodel, &serviceiter, 4, &serviceinfo, -1);
    start_service(serviceinfo->fullname);
  }
}

void on_restart_clicked(GtkToolButton *toolbutton, gpointer user_data) {
  GtkWidget *service_list = glade_xml_get_widget(xml, "service_list");
  GtkTreeIter serviceiter;
  GtkTreeSelection *selected;
  GtkTreeModel *servicemodel;
  struct s_info *serviceinfo;

  selected = gtk_tree_view_get_selection(GTK_TREE_VIEW(service_list));
  if(gtk_tree_selection_get_selected(selected, &servicemodel, &serviceiter)) {
    gtk_tree_model_get (servicemodel, &serviceiter, 4, &serviceinfo, -1);
    restart_service(serviceinfo->fullname);
  }
}

void on_zap_clicked(GtkToolButton *toolbutton, gpointer user_data) {
  GtkWidget *service_list = glade_xml_get_widget(xml, "service_list");
  GtkTreeIter serviceiter;
  GtkTreeSelection *selected;
  GtkTreeModel *servicemodel;
  struct s_info *serviceinfo;

  selected = gtk_tree_view_get_selection(GTK_TREE_VIEW(service_list));
  if(gtk_tree_selection_get_selected(selected, &servicemodel, &serviceiter)) {
    gtk_tree_model_get(servicemodel, &serviceiter, 4, &serviceinfo, -1);
    zap_service(serviceinfo->fullname);
  }
}

void on_edit_clicked(GtkToolButton *toolbutton, gpointer user_data) {
  GtkWidget *service_list = glade_xml_get_widget(xml, "service_list");
  on_service_list_row_activated(GTK_TREE_VIEW(service_list), NULL, NULL, NULL);
}

void on_runlevel_filter_combo_changed(GtkComboBox *combobox, gpointer user_data) {
  GtkTreeView *service_list = GTK_TREE_VIEW(glade_xml_get_widget(xml, "service_list"));
  GtkTreeModelSort *sortmodel = GTK_TREE_MODEL_SORT(gtk_tree_view_get_model(service_list));
  GtkTreeModelFilter *filtermodel = GTK_TREE_MODEL_FILTER(gtk_tree_model_sort_get_model(sortmodel));

  GtkWidget *start_button = glade_xml_get_widget(xml, "start_button");
  GtkWidget *stop_button = glade_xml_get_widget(xml, "stop_button");
  GtkWidget *restart_button = glade_xml_get_widget(xml, "restart_button");
  GtkTreeSelection *selected = gtk_tree_view_get_selection(service_list);
  gtk_tree_model_filter_refilter(filtermodel);
  if(!gtk_tree_selection_get_selected(selected, NULL, NULL)) {
    gtk_widget_set_sensitive(start_button, FALSE);
    gtk_widget_set_sensitive(stop_button, FALSE);
    gtk_widget_set_sensitive(restart_button, FALSE);
  }
}

void on_button_expand_clicked(GtkToolButton *toolbutton,
			      gpointer user_data) {
  GtkWidget *service_list = glade_xml_get_widget(xml, "service_list");
  gtk_tree_view_expand_all(GTK_TREE_VIEW(service_list));
}

void on_button_collapse_clicked(GtkToolButton *toolbutton, 
				gpointer user_data) {
  GtkWidget *service_list = glade_xml_get_widget(xml, "service_list");
  gtk_tree_view_collapse_all(GTK_TREE_VIEW(service_list));
}
